@@ -1,21 +1,20 @@
-/***************************************************************************
- *   Copyright (C) 2007-2009 by Lothar May                                 *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
- ***************************************************************************/
+/*****************************************************************************
+ * PokerTH - The open source texas holdem engine                             *
+ * Copyright (C) 2006-2011 Felix Hammer, Florian Thauer, Lothar May          *
+ *                                                                           *
+ * This program is free software: you can redistribute it and/or modify      *
+ * it under the terms of the GNU Affero General Public License as            *
+ * published by the Free Software Foundation, either version 3 of the        *
+ * License, or (at your option) any later version.                           *
+ *                                                                           *
+ * This program is distributed in the hope that it will be useful,           *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
+ * GNU Affero General Public License for more details.                       *
+ *                                                                           *
+ * You should have received a copy of the GNU Affero General Public License  *
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.     *
+ *****************************************************************************/
 /* Network server lobby thread. */
 
 #ifndef _SERVERLOBBYTHREAD_H_
@@ -23,6 +22,7 @@
 
 #include <boost/asio.hpp>
 #include <boost/enable_shared_from_this.hpp>
+#include <boost/uuid/uuid_generators.hpp>
 
 #include <net/sessionmanager.h>
 #include <net/netpacket.h>
@@ -30,14 +30,11 @@
 #include <gui/guiinterface.h>
 #include <gamedata.h>
 
-class Log;
-
 #define NET_LOBBY_THREAD_TERMINATE_TIMEOUT_MSEC		20000
 #define NET_ADMIN_IRC_TERMINATE_TIMEOUT_MSEC		4000
 
 
 class SenderHelper;
-class ReceiverHelper;
 class InternalServerCallback;
 class ServerIrcBotCallback;
 class ServerGame;
@@ -54,18 +51,17 @@ class ServerLobbyThread : public Thread, public boost::enable_shared_from_this<S
 {
 public:
 	ServerLobbyThread(GuiInterface &gui, ServerMode mode, ServerIrcBotCallback &ircBotCb, ConfigFile &serverConfig, AvatarManager &avatarManager,
-		boost::shared_ptr<boost::asio::io_service> ioService);
+					  boost::shared_ptr<boost::asio::io_service> ioService);
 	virtual ~ServerLobbyThread();
 
 	void Init(const std::string &logDir);
 	virtual void SignalTermination();
 
 	void AddConnection(boost::shared_ptr<boost::asio::ip::tcp::socket> sock);
-	void ReAddSession(SessionWrapper session, int reason);
-	void MoveSessionToGame(ServerGame &game, SessionWrapper session);
-	void RemoveSessionFromGame(SessionWrapper session);
-	void SessionError(SessionWrapper session, int errorCode);
-	void ResubscribeLobbyMsg(SessionWrapper session);
+	void ReAddSession(boost::shared_ptr<SessionData> session, int reason, unsigned gameId);
+	void MoveSessionToGame(boost::shared_ptr<ServerGame> game, boost::shared_ptr<SessionData> session, bool autoLeave);
+	void SessionError(boost::shared_ptr<SessionData> session, int errorCode);
+	void ResubscribeLobbyMsg(boost::shared_ptr<SessionData> session);
 	void NotifyPlayerJoinedLobby(unsigned playerId);
 	void NotifyPlayerLeftLobby(unsigned playerId);
 	void NotifyPlayerJoinedGame(unsigned gameId, unsigned playerId);
@@ -74,19 +70,21 @@ public:
 	void NotifyStartingGame(unsigned gameId);
 	void NotifyReopeningGame(unsigned gameId);
 
-	void HandleGameRetrievePlayerInfo(SessionWrapper session, const PlayerInfoRequestMessage_t &playerInfoRequest);
-	void HandleGameRetrieveAvatar(SessionWrapper session, const AvatarRequestMessage_t &retrieveAvatar);
-	void HandleChatRequest(SessionWrapper session, const ChatRequestMessage_t &chatRequest);
+	void DispatchPacket(boost::shared_ptr<SessionData> session, boost::shared_ptr<NetPacket> packet);
+	void HandleGameRetrievePlayerInfo(boost::shared_ptr<SessionData> session, const PlayerInfoRequestMessage_t &playerInfoRequest);
+	void HandleGameRetrieveAvatar(boost::shared_ptr<SessionData> session, const AvatarRequestMessage_t &retrieveAvatar);
+	void HandleChatRequest(boost::shared_ptr<SessionData> session, const ChatRequestMessage_t &chatRequest);
 
 	bool KickPlayerByName(const std::string &playerName);
 	std::string GetPlayerIPAddress(const std::string &playerName) const;
 	std::string GetPlayerNameFromId(unsigned playerId) const;
 	void RemovePlayer(unsigned playerId, unsigned errorCode);
-
+	void MutePlayerInGame(unsigned playerId);
 
 	void SendGlobalChat(const std::string &message);
 	void SendGlobalMsgBox(const std::string &message);
 	void SendChatBotMsg(const std::string &message);
+	void SendChatBotMsg(unsigned gameId, const std::string &message);
 	void ReconnectChatBot();
 
 	void AddComputerPlayer(boost::shared_ptr<PlayerData> player);
@@ -98,9 +96,8 @@ public:
 	u_int32_t GetNextGameId();
 	ServerCallback &GetCallback();
 
-	void SetGameDBId(u_int32_t gameId, DB_id gameDBId);
-
 	AvatarManager &GetAvatarManager();
+	ChatCleanerManager &GetChatCleaner();
 
 	ServerStats GetStats() const;
 	boost::posix_time::ptime GetStartTime() const;
@@ -114,7 +111,7 @@ public:
 protected:
 
 	typedef std::deque<boost::shared_ptr<boost::asio::ip::tcp::socket> > ConnectQueue;
-	typedef std::list<SessionWrapper> SessionList;
+	typedef std::list<boost::shared_ptr<SessionData> > SessionList;
 	typedef std::list<SessionId> SessionIdList;
 	typedef std::map<SessionId, boost::timers::portable::microsec_timer> TimerSessionMap;
 	typedef std::map<unsigned, boost::shared_ptr<ServerGame> > GameMap;
@@ -129,33 +126,37 @@ protected:
 	void ClearAuthContext();
 	void InitChatCleaner();
 
-	void HandleRead(const boost::system::error_code &ec, SessionId sessionId, size_t bytesRead);
-	void HandlePacket(SessionWrapper session, boost::shared_ptr<NetPacket> packet);
-	void HandleNetPacketInit(SessionWrapper session, const InitMessage_t &initMessage);
-	void HandleNetPacketAuthClientResponse(SessionWrapper session, const AuthClientResponse_t &clientResponse);
-	void HandleNetPacketAvatarHeader(SessionWrapper session, unsigned requestId, const AvatarHeader_t &avatarHeader);
-	void HandleNetPacketUnknownAvatar(SessionWrapper session, unsigned requestId, const UnknownAvatar_t &unknownAvatar);
-	void HandleNetPacketAvatarFile(SessionWrapper session, unsigned requestId, const AvatarData_t &avatarData);
-	void HandleNetPacketAvatarEnd(SessionWrapper session, unsigned requestId, const AvatarEnd_t &avatarEnd);
-	void HandleNetPacketRetrievePlayerInfo(SessionWrapper session, const PlayerInfoRequestMessage_t &playerInfoRequest);
-	void HandleNetPacketRetrieveAvatar(SessionWrapper session, const AvatarRequestMessage_t &retrieveAvatar);
-	void HandleNetPacketCreateGame(SessionWrapper session, const std::string &password, const JoinNewGame_t &newGame);
-	void HandleNetPacketJoinGame(SessionWrapper session, const std::string &password, const JoinExistingGame_t &joinGame);
-	void HandleNetPacketChatRequest(SessionWrapper session, const ChatRequestMessage_t &chatRequest);
-	void HandleNetPacketRejectGameInvitation(SessionWrapper session, const RejectGameInvitationMessage_t &reject);
-	void AuthChallenge(SessionWrapper session, const std::string &secret);
-	void InitAfterLogin(SessionWrapper session);
-	void EstablishSession(SessionWrapper session);
-	void AuthenticatePlayer(SessionWrapper session);
+	void HandlePacket(boost::shared_ptr<SessionData> session, boost::shared_ptr<NetPacket> packet);
+	void HandleNetPacketInit(boost::shared_ptr<SessionData> session, const InitMessage_t &initMessage);
+	void HandleNetPacketAuthClientResponse(boost::shared_ptr<SessionData> session, const AuthClientResponse_t &clientResponse);
+	void HandleNetPacketAvatarHeader(boost::shared_ptr<SessionData> session, unsigned requestId, const AvatarHeader_t &avatarHeader);
+	void HandleNetPacketUnknownAvatar(boost::shared_ptr<SessionData> session, unsigned requestId, const UnknownAvatar_t &unknownAvatar);
+	void HandleNetPacketAvatarFile(boost::shared_ptr<SessionData> session, unsigned requestId, const AvatarData_t &avatarData);
+	void HandleNetPacketAvatarEnd(boost::shared_ptr<SessionData> session, unsigned requestId, const AvatarEnd_t &avatarEnd);
+	void HandleNetPacketRetrievePlayerInfo(boost::shared_ptr<SessionData> session, const PlayerInfoRequestMessage_t &playerInfoRequest);
+	void HandleNetPacketRetrieveAvatar(boost::shared_ptr<SessionData> session, const AvatarRequestMessage_t &retrieveAvatar);
+	void HandleNetPacketCreateGame(boost::shared_ptr<SessionData> session, bool autoLeave, const JoinNewGame_t &newGame);
+	void HandleNetPacketJoinGame(boost::shared_ptr<SessionData> session, bool autoLeave, const JoinExistingGame_t &joinGame);
+	void HandleNetPacketRejoinGame(boost::shared_ptr<SessionData> session, bool autoLeave, const RejoinExistingGame_t &rejoinGame);
+	void HandleNetPacketChatRequest(boost::shared_ptr<SessionData> session, const ChatRequestMessage_t &chatRequest);
+	void HandleNetPacketRejectGameInvitation(boost::shared_ptr<SessionData> session, const RejectGameInvitationMessage_t &reject);
+	// TODO would be better to use state pattern here.
+	void AuthChallenge(boost::shared_ptr<SessionData> session, const std::string &secret);
+	void CheckAvatarBlacklist(boost::shared_ptr<SessionData> session);
+	void AvatarBlacklisted(unsigned playerId);
+	void AvatarOK(unsigned playerId);
+	void InitAfterLogin(boost::shared_ptr<SessionData> session);
+	void EstablishSession(boost::shared_ptr<SessionData> session);
+	void AuthenticatePlayer(boost::shared_ptr<SessionData> session);
 	void UserValid(unsigned playerId, const DBPlayerData &dbPlayerData);
 	void UserInvalid(unsigned playerId);
 	void UserBlocked(unsigned playerId);
+
 	void SendReportAvatarResult(unsigned byPlayerId, unsigned reportedPlayerId, bool success);
-	void RequestPlayerAvatar(SessionWrapper session);
+	void RequestPlayerAvatar(boost::shared_ptr<SessionData> session);
 	void TimerRemoveGame(const boost::system::error_code &ec);
 	void TimerRemovePlayer(const boost::system::error_code &ec);
 	void TimerUpdateClientLoginLock(const boost::system::error_code &ec);
-	void TimerCheckSessionTimeouts(const boost::system::error_code &ec);
 	void TimerCleanupAvatarCache(const boost::system::error_code &ec);
 
 	bool IsGameNameInUse(const std::string &gameName) const;
@@ -163,15 +164,16 @@ protected:
 	void InternalAddGame(boost::shared_ptr<ServerGame> game);
 	void InternalRemoveGame(boost::shared_ptr<ServerGame> game);
 	void InternalRemovePlayer(unsigned playerId, unsigned errorCode);
-	void InternalResubscribeMsg(SessionWrapper session);
+	void InternalMutePlayerInGame(unsigned playerId);
+	void InternalResubscribeMsg(boost::shared_ptr<SessionData> session);
 
-	void HandleReAddedSession(SessionWrapper session);
+	void HandleReAddedSession(boost::shared_ptr<SessionData> session);
 
-	void InternalCheckSessionTimeouts(SessionWrapper session);
+	void SessionTimeoutWarning(boost::shared_ptr<SessionData> session, unsigned remainingSec);
 
 	void CleanupSessionMap();
 
-	void CloseSession(SessionWrapper session);
+	void CloseSession(boost::shared_ptr<SessionData> session);
 	void SendError(boost::shared_ptr<SessionData> s, int errorCode);
 	void SendJoinGameFailed(boost::shared_ptr<SessionData> s, unsigned gameId, int reason);
 	void SendPlayerList(boost::shared_ptr<SessionData> s);
@@ -182,26 +184,25 @@ protected:
 	void ReadStatisticsFile();
 	void TimerSaveStatisticsFile(const boost::system::error_code &ec);
 
-	ReceiverHelper &GetReceiver();
-
 	InternalServerCallback &GetSenderCallback();
 	GuiInterface &GetGui();
 	ServerIrcBotCallback &GetIrcBotCallback();
 
-	bool IsPlayerConnected(const std::string &name) const;
+	unsigned GetPlayerId(const std::string &name) const;
 
 	static boost::shared_ptr<NetPacket> CreateNetPacketPlayerListNew(unsigned playerId);
 	static boost::shared_ptr<NetPacket> CreateNetPacketPlayerListLeft(unsigned playerId);
 	static boost::shared_ptr<NetPacket> CreateNetPacketGameListNew(const ServerGame &game);
 	static boost::shared_ptr<NetPacket> CreateNetPacketGameListUpdate(unsigned gameId, GameMode mode);
 
+	u_int32_t GetRejoinGameIdForPlayer(const std::string &playerName, const std::string &guid, unsigned &outPlayerUniqueId);
+
 private:
 
 	boost::shared_ptr<boost::asio::io_service> m_ioService;
 
 	boost::shared_ptr<InternalServerCallback> m_internalServerCallback;
 	boost::shared_ptr<SenderHelper> m_sender;
-	boost::shared_ptr<ReceiverHelper> m_receiver;
 
 	SessionManager m_sessionManager;
 	SessionManager m_gameSessionManager;
@@ -214,9 +215,6 @@ private:
 	RemoveGameList m_removeGameList;
 	mutable boost::mutex m_removeGameListMutex;
 
-	RemovePlayerList m_removePlayerList;
-	mutable boost::mutex m_removePlayerListMutex;
-
 	PlayerDataMap m_computerPlayers;
 	mutable boost::mutex m_computerPlayersMutex;
 
@@ -235,8 +233,6 @@ private:
 	u_int32_t m_curSessionId;
 	mutable boost::mutex m_curUniquePlayerIdMutex;
 
-	boost::shared_ptr<Log> m_serverLog;
-
 	ServerStats m_statData;
 	bool m_statDataChanged;
 	mutable boost::mutex m_statMutex;
@@ -246,15 +242,14 @@ private:
 	boost::shared_ptr<ServerDBInterface> m_database;
 
 	boost::asio::deadline_timer m_removeGameTimer;
-	boost::asio::deadline_timer m_removePlayerTimer;
-	boost::asio::deadline_timer m_sessionTimeoutTimer;
-	boost::asio::deadline_timer m_avatarCleanupTimer;
 	boost::asio::deadline_timer m_saveStatisticsTimer;
 	boost::asio::deadline_timer m_loginLockTimer;
 
+	boost::uuids::random_generator m_sessionIdGenerator;
+
 	const boost::posix_time::ptime m_startTime;
 
-friend class InternalServerCallback;
+	friend class InternalServerCallback;
 };
 
 #endif
